@@ -1416,6 +1416,99 @@ where C: Cursor, T: BitStore {
 		})
 	}
 
+	/// Performs “reverse” addition (left to right instead of right to left).
+	///
+	/// This addition traverses the addends from left to right, performing
+	/// the addition at each index and writing the sum into `self`.
+	///
+	/// If `addend` expires before `self` does, `addend` is zero-extended and
+	/// the carry propagates through the rest of `self`. If `self` expires
+	/// before `addend`, then `self` is zero-extended and the carry propagates
+	/// through the rest of `addend`, growing `self` until `addend` expires.
+	///
+	/// An infinite `addend` will cause unbounded memory growth until the vector
+	/// overflows and panics.
+	///
+	/// # Parameters
+	///
+	/// - `self`
+	/// - `addend: impl IntoIterator<Item=bool>`: A stream of bits to add into
+	///   `self`, from left to right.
+	///
+	/// # Returns
+	///
+	/// The sum vector of `self` and `addend`.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::prelude::*;
+	///
+	/// let a = bitvec![0, 1, 0, 1];
+	/// let b = bitvec![0, 0, 1, 1];
+	/// let c = a.add_reverse(b);
+	/// assert_eq!(c, bitvec![0, 1, 1, 0, 1]);
+	/// ```
+	pub fn add_reverse<I>(mut self, addend: I) -> Self
+	where I: IntoIterator<Item=bool> {
+		self.add_assign_reverse(addend);
+		self
+	}
+
+	/// Performs “reverse” addition (left to right instead of right to left).
+	///
+	/// This addition traverses the addends from left to right, performing
+	/// the addition at each index and writing the sum into `self`.
+	///
+	/// If `addend` expires before `self` does, `addend` is zero-extended and
+	/// the carry propagates through the rest of `self`. If `self` expires
+	/// before `addend`, then `self` is zero-extended and the carry propagates
+	/// through the rest of `addend`, growing `self` until `addend` expires.
+	///
+	/// An infinite `addend` will cause unbounded memory growth until the vector
+	/// overflows and panics.
+	///
+	/// # Parameters
+	///
+	/// - `&mut self`
+	/// - `addend: impl IntoIterator<Item=bool>`: A stream of bits to add into
+	///   `self`, from left to right.
+	///
+	/// # Effects
+	///
+	/// `self` may grow as a result of the final carry-out bit being `1` and
+	/// pushed onto the right end.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::prelude::*;
+	///
+	/// let mut a = bitvec![0, 1, 0, 1];
+	/// let     b = bitvec![0, 0, 1, 1];
+	/// a.add_assign_reverse(&b);
+	/// assert_eq!(a, bitvec![0, 1, 1, 0, 1]);
+	/// ```
+	pub fn add_assign_reverse<I>(&mut self, addend: I)
+	where I: IntoIterator<Item=bool> {
+		//  Set up iteration over the addend
+		let mut addend = addend.into_iter().fuse();
+		//  Delegate to the `BitSlice` implementation for the initial addition.
+		//  If `addend` expires first, it zero-extends; if `self` expires first,
+		//  `addend` will still have its remnant for the next stage.
+		let mut c = self.as_mut_bitslice().add_assign_reverse(addend.by_ref());
+		//  If `addend` still has bits to provide, zero-extend `self` and add
+		//  them in.
+		for b in addend {
+			let (y, z) = crate::rca1(false, b, c);
+			self.push(y);
+			c = z;
+		}
+		if c {
+			self.push(true);
+		}
+	}
+
 	/// Changes the cursor type on the vector handle, without changing its
 	/// contents.
 	///
@@ -2229,11 +2322,7 @@ where C: Cursor, T: BitStore {
 		let mut stack = BitVec::<C, T>::with_capacity(self.len());
 		let addend = addend.into_iter().rev().chain(repeat(false));
 		for (a, b) in self.iter().rev().zip(addend) {
-			//  See `<BitSlice as AddAssign>::add_assign`.
-			static JUMP: [u8; 8] = [0, 2, 2, 1, 2, 1, 1, 3];
-			let jmp = ((c as u8) << 2) | ((a as u8) << 1) | (b as u8);
-			let yz = JUMP[jmp as usize];
-			let (y, z) = (yz & 2 != 0, yz & 1 != 0);
+			let (y, z) = crate::rca1(a, b, c);
 			stack.push(y);
 			c = z;
 		}
